##
# Copyright (c) 2020 - 2022 Xilinx, Inc.  All rights reserved.
# Copyright (c) 2022 - 2023 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
##
##  @logg.py
#   This class contains print methods for different levels of log
#
from config_app import *

class Logg:
    ##  @Enum for log type
    #
    DEBUG = 0
    ERROR = 1
    RELEASE = 2

    ##  @def printlog(logtype, log)
    #   Private function prints the log with extended log type name to standard out.
    #   @param logtype     log type enums. eg: DEBUG, ERROR, RELEASE
    #   @param log         log message to print
    #
    @staticmethod
    def __printlog(logtype,log):
        pref = logtype
        if len(logtype):
            pref = pref + " : "
        print(pref+str(log))

    ##  @def degug(logstr, loglevel)
    #   This method prints logs based on indexing levels.
    #   @param logsrt          log message to print
    #   @param loglevel        log levels enum properties, DEBUG, ERROR, RELEASE
    #
    @staticmethod
    def log(logstr, loglevel = RELEASE):
        if loglevel == Logg.DEBUG and app_config["deployment"] == "DEBUG":
            Logg.__printlog("DEBUG",logstr)
        if loglevel == Logg.ERROR:
            Logg.__printlog("ERROR", logstr)
        if loglevel == Logg.RELEASE:
            Logg.__printlog("", logstr)



##
# Copyright (c) 2020 - 2022 Xilinx, Inc.  All rights reserved.
# Copyright (c) 2022 - 2023 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
##

##  @term.py
#   This class contains methods that interact with terminals and returns results.
#
import subprocess
import threading

from logg import *

term_mutex = threading.Lock()
class Term:
    ##  @def exec_cmd(cmd)
    #   function to execute cmd on terminal and returns the reuslt.
    #   @param cmd          command to execute on terminal.
    #   @return             result of cmd on sucess
    #                       None on failure
    #
    @staticmethod
    def exec_cmd(cmd):
        global term_mutex
        term_mutex.acquire()
        Logg.log("------ cmd locked",Logg.DEBUG)
        try:
            
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,shell=True)
            outs, errs = proc.communicate()
            if errs is None:
                Logg.log(cmd,Logg.DEBUG)
                Logg.log(outs,Logg.DEBUG)
                Logg.log("====== cmd lock released",Logg.DEBUG)
                term_mutex.release()
                #return outs.decode('utf-8')
                try:
                    return outs.decode('utf-8')
                except :
                    return outs.decode('iso-8859-1')
            else:
                Logg.log("error",Logg.DEBUG)
                term_mutex.release()
                return None
        except FileNotFoundError:
            Logg.log("error 2",Logg.DEBUG)
            term_mutex.release()
            return None

class Xsdb:
    ##  @def exec_cmd(cmd)
    #   function to execute cmd on Xsdb and returns the reuslt.
    #   @param cmd          command to execute on terminal.
    #   @return             result of cmd on sucess
    #                       None on failure
    #
    @staticmethod
    def exec_cmd(cmd):
        try:
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            outs, errs = proc.communicate()
            if errs is None:
                return outs.decode()
            else:
                return None
        except FileNotFoundError:
            return None

class Other:
    @staticmethod
    def exec_cmd(cmd):
        return cmd

##  Factory class that initializes/calls the system calls related classes
#
class SysFactory:
    ## Enums to specify class type
    #
    TERMINAL = "Term"
    XSDB = "Xsdb"

    ##  @def exec_cmd(cmd,cmdtype)
    #   function to execute cmd based on type and returns the reuslt.
    #   @param cmd          command to execute.
    #   @param cmdtype      platform type on which cmd has to be executed
    #   @return             result of cmd on sucess
    #                       None on failure
    #
    @staticmethod
    def exec_cmd(command, cmdType=None):
        if cmdType == SysFactory.TERMINAL:
            return Term.exec_cmd(command)
        elif cmdType == SysFactory.XSDB:
            return Xsdb.exec_cmd(command)
        else:
            return Other.exec_cmd(command)



##
# Copyright (c) 2020 - 2022 Xilinx, Inc.  All rights reserved.
# Copyright (c) 2022 - 2024 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
##
from config_app import *

class Parse:
    def parse_cmd_resp(self, data, component,targ="",params = ""):
        if(component == "getpower" or component == "getcalpower"):
            return self.parseGetPower(data)
        elif(component == "getINA226"):
            return self.parseGetINA226(data)
        elif(component.startswith("list")):
            return self.parseList(data)
        elif((component == "getvoltage" or component == "powerdomain") and ("all" not in params)):
            return self.parseGetVoltage(data)
        elif(component == "getclock"):
            return self.parseGetClock(data,targ)
        elif(component == "getmeasuredclock"):
            return self.parseGetMeasuredClock(data,targ)
        elif(component == "BIT"):
            return self.parseBit(data)
        elif(component == "getddr"):
            return self.parseDDR(data)
        elif(component == "version"):
            return self.parseversion(data)
        elif(component == "getgpio"):
            return self.parsegpio(data)
        elif(component == "geteeprom" and 'summary' in params):
            return self.dashboard_eeprom(data)
        elif(component == "getioexp" or component == "getpwmSFP" or component == "getSFP" or component == "getQSFP"
or component == "getpwmQSFP" or component == "getpwmoQSFP" or component == "getEBM" or component == "getFMC"
or component == "geteeprom" or component == "getvoltage"):
            return self.parseioexp(data)
        else:
            return ""
    def temperature(self,data):
        # Parse temperature data from data.
        pass

    def dashboard_eeprom(self,data):
        # Parse eeprom data for details
        pass

import json
class ParseData(Parse):
    def temperature(self,data):
        # Parse temperature data from data.
        obj = data.strip().split(":")
        if 'ERROR' in data:
            return {"temp":"-"}
        else:
            return {"temp":obj[1]}

    def parseGetINA226(self,data):
        # Parse eeprom data for details
        resar = data.rstrip().split("\n")
        res = {"Configuration":"-"
               ,"Shunt_Voltage":"-"
               ,"Bus_Voltage":"-"
               ,"Power":"-"
               ,"Current":"-"
               ,"Calibration":"-"
               ,"Mask_Enable":"-"
               ,"Alert_Limit": "-"
               ,"Die_ID":"-"
                }
        for re in resar:
            ary = re.split(":")
            print(ary)
            if ary[0].startswith('Configuration'):
                res['Configuration']=ary[1].strip()
            if ary[0].startswith('Shunt Voltage'):
                res["Shunt_Voltage"]=ary[1].strip()
            if ary[0].startswith('Bus Voltage'):
                res["Bus_Voltage"]=ary[1].strip()
            if ary[0].startswith('Power'):
                res["Power"]=ary[1].strip()
            if ary[0].startswith('Current'):
                res["Current"]=ary[1].strip()
            if ary[0].startswith('Calibration'):
                res["Calibration"]=ary[1].strip()
            if ary[0].startswith('Mask/Enable'):
                res["Mask_Enable"]=ary[1].strip()
            if ary[0].startswith('Alert Limit'):
                res["Alert_Limit"]=ary[1].strip()
            if ary[0].startswith('Die ID'):
                res["Die_ID"]=ary[1].strip()
        return res
    def dashboard_eeprom(self,data):
        # Parse eeprom data for details
        ver = "" + app_config["major_version"]+"."+app_config["minor_version"]
        if app_config["deployment"] == "DEBUG":
            ver = ver + "." + app_config["dev_for_major_ver"]+"."+app_config["dev_minor_ver"] 
        resar = data.rstrip().split("\n")
        res = {}
        for re in resar:
            ary = re.split(":",1)
            if ary[0].startswith('Language') or ary[0].startswith('Manufacturing Date'):
                continue
            res[ary[0]] = ary[1].strip()
            print(res[ary[0]])
        f_res = {}
        f_res["summary"] = res
        f_res["appversion"] = ver
        return f_res  
        return res
    def parseGetPower(self,data):
        resar = data.rstrip().split("\n")
        res = {}
        for re in resar:
            ary = re.split(":")
            if ary[0].startswith('Voltage'):
                res["voltage"]=ary[1].strip()
            if ary[0].startswith('Current'):
                res["current"]=ary[1].strip()
            if ary[0].startswith('Power'):
                res["power"]=ary[1].strip()
        return res
    def parseBit(self,data):
        resar = data.strip().split(":")
        res = {}
        if len(resar) > 1:
            res["state"] = resar[1]
        else:
            res["state"] = resar[0]
        res["message"] = data.strip()
        return res
    def parseDDR(self,data):
        resar = data.strip() #.split(":")
        res = {}
        if(len(data.splitlines()) > 1):
            resar = resar.replace("\n","</br>")
            res["info"] = resar
        else:
            res["temp"] = resar.split(":")[1]
        return res
    def parseversion(self,data):
        resar = data.strip().split('\n')[0].split(":")
        res = {}
        if data.startswith('Error'):
            res["version"] = '-'
        else:
            res["version"] = resar[1].strip()
        return res
    def parseGetVoltage(self,data):
        resar = data.strip().split(":")
        res = {}
        res["voltage"] = resar[1]
        return res
    def parseGetClock(self,data,targ):
        res = {}
        if targ.startswith('8A34001'):
            res["frequency"] = data.replace("\n","</br>")
        else:
            resar = data.strip().split(":")
            res["frequency"] = resar[1]
        return res
    def parseGetMeasuredClock(self,data,targ):
        res = {}
        if targ.startswith('8A34001'):
            res["measuredfrequency"] = data.replace("\n","</br>")
        else:
            resar = data.strip().split(":")
            if len(resar) > 1:
                res["measuredfrequency"] = resar[1]
            else:
                res["measuredfrequency"] = "-"
        return res
    def parsegpio(self,data):
        resar = data.strip().split(":")
        res = {}
        res["gpio"] = resar[1]
        return res
    def parseioexp(self,data):
        resar = data.strip().replace("\n","</br>")
        res = {}
        res["io"] = resar
        return res
    def parseList(self,data):
        res = data.rstrip().split("\n")
        return res

import random
class ParseDataStatic(Parse):
    def temperature(self,data):
        # Parse temperature data from data.
        temp = random.randrange(16,19)
        return {"temp":temp}

    def dashboard_eeprom(self,data):
        # Parse eeprom data for details
        ver = "" + app_config["major_version"]+"."+app_config["minor_version"]
        if app_config["deployment"] == "DEBUG":
            ver = ver + "." + app_config["dev_for_major_ver"]+"."+app_config["dev_minor_ver"] 

        res = {"device":"VCK190"
               ,"sil_rev":"1.0"
               ,"board_pn":"0"
               ,"rev":"-"
               ,"serial_number":"abcd1234"
               ,"mac1":"0123456789"
               ,"mac2":"1234567890"
               ,"appversion": ver
                }
        return res
    def parseGetPower(self,data):
        res = {
            "power":4
            ,"voltage":1.9
            ,"current":2.5
        }
        return res
    def parseList(self,data):
        res = ["a","b","c"]
        return res



##
# Copyright (c) 2020 - 2022 Xilinx, Inc.  All rights reserved.
# Copyright (c) 2022 - 2023 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
##
import os
import json
from config_app import *
from term import *
import psutil

def checkJNK():
    kercount = 0
    for proc in psutil.process_iter():
        if "python3" in proc.name() and "ipykernel_launcher" in proc.cmdline():
            kercount = kercount + 1
    return kercount

def checkJNS():
    kcount = 0
    for proc in psutil.process_iter():
        if "python3" in proc.name() and "/usr/bin/jupyter-notebook" in proc.cmdline():
            kcount = kcount + 1
    return kcount


def jnurl():
    
    jnlst = ""
    jnd = ""
    ur = ""
    deviname = Term.exec_cmd(app_config["sc_app_path"]+" -c board\n")    
    if "VCK" in deviname or "VMK" in deviname:
        jnlst = os.listdir(app_config["jnlocalrundirroot"])
        jnd=app_config["jnlocalrundirroot"]
    else:
        jnlst = os.listdir(app_config["jnlocalrundir"])
        jnd=app_config["jnlocalrundir"]

    nbfiles = [jnd+"/"+f for f in jnlst if "nbserver" in f and ".json" in f]
    
    latfile = ""
    try:
        latfile = max(nbfiles, key=os.path.getctime)
    except:
        latfile = ""

    if len(latfile) == 0 or checkJNS() == 0:
        return (0, "No Active Jupyter notebook session available")
    try:
        nbservfile = latfile
        f = open(nbservfile,"r")
        data = json.load(f)
        ur = ur + data["url"]+"?token="+data["token"]
        f.close()
    except:
        print("failed")
    return (1, ur)




##
# Copyright (c) 2020 - 2022 Xilinx, Inc.  All rights reserved.
# Copyright (c) 2022 - 2024 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
##

##  @systemcontroller.py
#   This file contains main. Initiates and starts server.
#

# imports
from flask import Flask, render_template, request
from flask import Response  ,jsonify
from flask_restful import Resource, Api , reqparse
from werkzeug.utils import secure_filename
import shutil

from logg import *
from restserv import *
from parse import *
from config_app import *
from jnservice import *
import restserv
##  Main that calls other functions and launches the server.
#
#
app = Flask(__name__)
api = Api(app)
app.config['MAX_CONTENT_LENGTH'] = 1 * 1024 * 1024 * 1024 # 1 GB
ALLOWED_CLK_EXTENSIONS = set(app_config["allowed_clock_files"])
# ALLOWED_PDI_EXTENSIONS = set(app_config["allowed_pdi_files"])

def allowed_clk_file(filename):
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in ALLOWED_CLK_EXTENSIONS
def allowed_pdi_file(filename):
    return True
def allowed_rauc_file(filename):
    return True

@app.route('/')
def index():
    if not os.path.exists(app_config["uploaded_files_path"]):
        os.makedirs(app_config["uploaded_files_path"])
    if not os.path.exists(app_config["PDIFilePath"]):
        os.makedirs(app_config["PDIFilePath"])
    # returning template.
    generate_gen_sc_file(sc_app_path, app_config)
    return render_template("index.html", versioning = ""+ app_config["major_version"]+"."+app_config["minor_version"] + "." + app_config["dev_for_major_ver"]+"."+app_config["dev_minor_ver"])	
## Resources
api.add_resource(Poll,"/poll")
api.add_resource(FuncReq,"/funcreq")
api.add_resource(CmdQuery,"/cmdquery")
api.add_resource(MultiCmdQuery,"/multicmdquery")
api.add_resource(EEPROMDetails,"/eeprom_details")
api.add_resource(ClockFilesList,"/clock_files")
api.add_resource(Banner,"/notif")
api.add_resource(ScriptRunner, "/scriptrunner")
api.add_resource(InstallBoard, "/installboard")
api.add_resource(exportCSV, "/exportcsv")
api.add_resource(RaucUpdate, "/raucupdate")
## Resources
import threading
req_lock = threading.Lock()

@app.before_request
def lock_next_requests():
    #if  "cmdquery" in request.path or "eeprom_details" in request.path or "funcreq" in request.path:
    #    if checkJNK() >= 1:
    #        return {"key":"A jupyter notebook kernel is running"},204
    if  "cmdquery" in request.path or "eeprom_details" in request.path or "funcreq" in request.path:
        global req_lock
        req_lock.acquire()
        Logg.log("************* Request locked",Logg.DEBUG)
@app.after_request
def unlock_for_next_req(response):
    global req_lock
    if req_lock.locked():
        req_lock.release()
        Logg.log("************* Released lock",Logg.DEBUG)
    return response

def crstring():
    return '''/*
* Copyright (c) 2020 - 2022 Xilinx, Inc.  All rights reserved.
* Copyright (c) 2022 - 2024 Advanced Micro Devices, Inc.  All rights reserved.
*
* SPDX-License-Identifier: MIT
*/
'''
def generate_gen_sc_file(sc_app_path, app_config):
    # Creating a config file which contains a list of each tab type.
    f = open("./static/js/gen_sc.js", "w")
    p = ParseData()
    f.write(crstring())
    f.write("var listsjson_sc = {\n")
    listfeature = p.parse_cmd_resp(Term.exec_cmd(sc_app_path + " -c listfeature\n"), "listfeature")
    finStr = ""
    if "FMC" in listfeature:
        listfeature.append("FMCvoltage")
    for k in listfeature:
        if len(finStr):
            finStr = finStr + ","
        finStr = finStr + '"list' + k + '"'
    f.write('"listfeature":[' + finStr + "],\n")

    lisj = app_config["config_sc_list_cmds"]
    for ind, k in enumerate(listfeature):
        ke = "list" + k
        if ind > 0:
            f.write(",\n")
        response = Term.exec_cmd(sc_app_path + " -c " + ke + "\n")
        resp = p.parse_cmd_resp(response, ke)
        finStr = ""
        for k in resp:
            if len(finStr):
                finStr = finStr + ","
            finStr = finStr + '"' + k + '"'
        f.write('"' + ke + '":[' + finStr + "]")

    vers1 = ""+ app_config["major_version"]+"."+app_config["minor_version"]
    vers2 = ""+ p.parse_cmd_resp(Term.exec_cmd(sc_app_path+" -c version"),"version")["version"]
    f.write(',\n"version":"' + vers1 + '/' + vers2 + '"')
    lisk = app_config["config_bit_list_cmds"]
    for ind, k in enumerate(lisk):
        ke = "describeBIT"
        if ind > 0:
            f.write(",\n")
        response = Term.exec_cmd(sc_app_path + " -c " + k + "\n")
        resp = p.parse_cmd_resp(response, k)
        resp = [item.split(' - ')[0] for item in resp]
        finStr = ""
        for k in resp:
            k = '"' + k + '"'
            if len(finStr):
                finStr = finStr + ","
            finStr += '"' + Term.exec_cmd(sc_app_path + " -c " + ke + " -t " + k).strip() + '"'
        f.write(',\n"Description":[' + finStr + ']')
    tcsfiles = ""
    txtfiles = ""
    binfiles = ""
    for c in os.listdir(app_config["8A34001_clk_files_path"]):
        if c.endswith(".tcs"):
            if len(tcsfiles):
                tcsfiles = tcsfiles + ","
            tcsfiles = tcsfiles + '"' + c + '"'
        if c.endswith(".txt"):
            if len(txtfiles):
                txtfiles = txtfiles + ","
            txtfiles = txtfiles + '"' + c + '"'
        if c.endswith(".bin"):
            if len(binfiles):
                binfiles = binfiles + ","
            binfiles = binfiles + '"' + c + '"'

    f.write(',\n"8A34001_clk_tcs_files":[' + tcsfiles + ']')
    f.write(',\n"8A34001_clk_txt_files":[' + txtfiles + ']')
    f.write(',\n"8A34001_clk_bin_files":[' + binfiles + ']')

    f.write("\n};")
    f.close()

    # Check device
    deviname = Term.exec_cmd(sc_app_path + " -c board\n")
    app_config["deviname"] = deviname.strip()
    print("deviname = ", deviname)
    string_file = "./static/js/" + deviname.lower().strip() + "_strings.js"
    print("stringfile = ", string_file)
    if not os.path.exists(string_file):
        string_file = "./static/js/temp_strings.js"
    shutil.copyfile(string_file, "./static/js/beam_strings.js")

    f = open("./static/js/gen_sc.js", "a")
    f.write("\nvar general = {\n")
    deviname = deviname.strip('\n')
    f.write('"boardName":''"' + deviname + '"')
    f.write("\n}")
    f.close()

    # Read Version info
    f = open("./static/js/gen_sc.js", "a")
    f.write("\nvar version = {\n")
    version_info = Term.exec_cmd(app_config["versioninfo"].split('\n'))
    lines = version_info.strip().split('\n')
    f.write('    "version_info": [\n')
    for line in lines:
        line = line.replace('"', '\\"')
        f.write('        "{}",\n'.format(line))
    f.write("    ]\n};\n")
    f.close()
    
    # Check Rauc support
    f = open("./static/js/gen_sc.js", "a")
    f.write("var rauc_support = {\n")
    rauc_support = Term.exec_cmd("rauc")
    if "rauc: command not found" in rauc_support:
        rauc_status = "no"
    else:
        rauc_status = "yes"
    f.write('"rauc":''"' + rauc_status + '"')
    f.write("\n};")
    f.close()

    #   bit tab components
    f = open("./static/js/gen_bit.js", "w")
    p = ParseData()
    f.write(crstring())
    f.write("var listsjson_bit = {\n")
    lisj = app_config["config_bit_list_cmds"]
    for ind, ke in enumerate(lisj):
        if ind > 0:
            f.write(",\n")
        response = Term.exec_cmd(sc_app_path + " -c " + ke + "\n")
        resp = p.parse_cmd_resp(response, ke)
        finStr = ""
        for k in resp:
            if len(finStr):
                finStr = finStr + ","
            finStr = finStr + '"' + k + '"'
        f.write('"' + ke + '":[' + finStr + "]");

    f.write("\n}")
    f.close()
    #   boot mode list
    f = open("./static/js/gen_bm.js", "w")
    f.write(crstring())
    f.write("var listsjson_bm = {\n")
    lisj = app_config["config_bm_list_cmds"]
    for ind, ke in enumerate(lisj):
        if ind > 0:
            f.write(",\n")
        response = Term.exec_cmd(sc_app_path + " -c " + ke + "\n")
        resp = p.parse_cmd_resp(response, ke)
        finStr = ""
        for k in resp:
            if len(finStr):
                finStr = finStr + ","
            finStr = finStr + '"' + k + '"'
        f.write('"' + ke + '":[' + finStr + "]");

    f.write("\n}")
    f.close()

if __name__ == '__main__':
    Notif.notification_load()
    if not len(sc_app_path) or not len(SysFactory.exec_cmd("which " + sc_app_path, SysFactory.TERMINAL)):
        Logg.log("Please check sc_app Path", Logg.RELEASE)
        exit(0)
    # if not len(sensors_app) or not len(SysFactory.exec_cmd("which "+sensors_app,SysFactory.TERMINAL)):
    #    Logg.log("Please check sensors app Path",Logg.RELEASE)
    #    exit(0)

    generate_gen_sc_file(sc_app_path, app_config)
    @app.route('/uploader', methods=['POST'])
    def upload_file():
        req = request.args.get('func')
        if 'file' not in request.files:
            return jsonify({'message': 'No file part in the request'})

        files = request.files.getlist('file')

        errors = False
        for file in files:
            if file.filename == '':
                return jsonify({'message': 'No file selected for uploading'})

            if req == 'clock' and file and allowed_clk_file(file.filename):
                filename = secure_filename(file.filename)
                file.save(os.path.join(app_config["uploaded_files_path"], filename))
            elif req == 'pdi' and file and allowed_pdi_file(file.filename):
                filename = secure_filename(file.filename)
                file.save(os.path.join(app_config["PDIFilePath"], filename))
            elif req == 'rauc' and file and allowed_rauc_file(file.filename):
                filename = secure_filename(file.filename)
                file.save(os.path.join(app_config["raucFilepath"], filename))
            else:
                errors = True


        if errors:
            return jsonify({'message': 'Some files could not be uploaded. Allowed file types are txt'})

        return jsonify({'message': 'Files successfully uploaded'})
    app.run(host="0.0.0.0", port=80, debug=True)





##
# Copyright (c) 2020 - 2022 Xilinx, Inc.  All rights reserved.
# Copyright (c) 2022 - 2024 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
##


app_config = {
    "deployment":"RELEASE"              # RELEASE, DEBUG
    ,"major_version":"1"
    ,"minor_version" : "4"
    ,"dev_for_major_ver" : "1"
    ,"dev_minor_ver" : "3"
    ,"sc_app_path":"sc_app"
    ,"scriptfile":"/usr/bin/collect_logs.sh"
    ,"boardsetupfile":"/usr/bin/setup_board.sh"
    ,"versioninfo":"/usr/bin/version_info.sh" 
    ,"bitlogFilePath":"/usr/share/system-controller-app/.sc_app/BIT.log"
    ,"csvFIlePath":"/usr/share/raft/examples/python/pmtool/pm-cmd.py -c output-csv -d 10 -s 1 -o ./static/tmp/"
    ,"raucFilepath":"/data/"
    ,"config_sc_list_cmds":["listpower","listclock","listvoltage","listFMCvoltage","listgpio","listSFP","listpowerdomain","listQSFP","listFMC"]
    ,"config_bit_list_cmds":["listBIT"]
    ,"config_bm_list_cmds":["listbootmode"]
    ,"jnlocalrundir":"/home/petalinux/.local/share/jupyter/runtime/"
    ,"jnlocalrundirroot":"/home/root/.local/share/jupyter/runtime/"
    ,"8A34001_clk_files_path":"/usr/share/system-controller-app/BIT/clock_files/"
    ,"board_file_path":"/home/root/.sc_app/board"
    ,"uploaded_files_path":"/data/clock_files/"
    ,"PDIFilePath":"/data/PDIs/"
    ,"allowed_clock_files":['txt', 'tcs', 'bin']
}




##
# Copyright (c) 2020 - 2022 Xilinx, Inc.  All rights reserved.
# Copyright (c) 2022 - 2024 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
##
from flask import render_template
from flask_restful import Resource, request
from enum import Enum
from config_app import *
from term import *
from parse import *
from jnservice import *
##
# TODO :: Change parse data static to dynamic class for realtime data.
#parse = ParseDataStatic()
parse = ParseData()
deviname = ""

sc_app_path = app_config["sc_app_path"]
class BootMode:
    active_bootmode = "-"
    @staticmethod
    def getActiveBootMode():
        return BootMode.active_bootmode
    @staticmethod
    def setBootMode(mode):
        BootMode.active_bootmode = mode
        SysFactory.exec_cmd(sc_app_path +" -c setbootmode -t "+mode + " -v alternate",SysFactory.TERMINAL) 
        res = SysFactory.exec_cmd(sc_app_path +" -c reset",SysFactory.TERMINAL)           
        if res.startswith("ERROR") or "ERROR" in res:
            resp_json = {
                "status":"error"
                ,"data":res
            }
        else :
            resp_json = {
                "status":"success"
                ,"data":res
            } 
        return resp_json
class ReqFunctions:
    global sc_app_path
    @staticmethod
    def polls(params):
        gtemp_targ = ""
        if len(params) > 0:
            gtemp_targ = params[0]
        try:
            result = {"temp":"-"}
            stat = "error"
            if checkJNK() == 0:
                stat = "success"
                response = Term.exec_cmd(sc_app_path+" -c gettemp -t "+gtemp_targ)
                result = parse.temperature(response)
            result["active_bootmode"] = BootMode.getActiveBootMode()
            resp_json = {
                "status":stat
                ,"data":result
            }
            return resp_json,200
        except Exception as e:
            resp_json = {
                "status":"error"
                ,"data":{"error":"%s"%e}
            }
            return resp_json,500
    def bootmode_set(mode):
        res = BootMode.setBootMode(mode);
        return res,200
    def jnlink():
        jnu = jnurl()
        return {"status":"success","data":jnu},200
class FuncReq(Resource):
    def get(self,):
        req = request.args.get('func')
        tar = request.args.get('params')
        params_req = request.args.get('params')
        params = params_req.split(",")
        
        if req.startswith('poll'):
            
            return ReqFunctions.polls(params)
        if req.startswith('jnlink'):
            return ReqFunctions.jnlink()
        if req.startswith('setbootmode'):
            if checkJNK() >= 1: 
                resp_json = { 
                    "status":"error"
                    ,"data": "Notebook kernel is running. Please stop running kernel."
                }                       
                return resp_json,200

            if len(params):
                return ReqFunctions.bootmode_set(params[0])
        resp_json = {
            "status":"error"
            ,"data":{"error":"Fail"}
        }
        return resp_json,500
class Poll(Resource):
    def get(self,):
        try:
            if checkJNK() >= 1: 
                resp_json = { 
                    "status":"error"
                    ,"data": "Notebook kernel is running. Please stop running kernel."
                }                       
                return resp_json,200
            response = Term.exec_cmd(sc_app_path+" -c gettemp -t MDIO")
            result = parse.temperature(response)
            result["active_bootmode"] = "jtag"
            resp_json = {
                "status":"success"
                ,"data":result
            }
            return resp_json,200
        except Exception as e:
            resp_json = {
                "status":"error"
                ,"data":{"error":"%s"%e}
            }
            return resp_json,500
class EEPROMDetails(Resource):
    def get(self,):
        try:
            response = ""
            if checkJNK() == 0: 
                response = Term.exec_cmd(sc_app_path+" -c eeprom")
            result = parse.dashboard_eeprom(response)
            resp_json = {
                "status":"success"
                ,"data":result
            }
            return resp_json,200
        except Exception as e:
            resp_json = {
                "status":"error"
                ,"data":{"error":"%s"%e}
            }
            return resp_json,500
class ClockFilesList(Resource):
    def get(self,):
        try:
            req = request.args.get('func')
            if req == "clock":
                tcs_files = []
                txt_files = []
                bin_files = []
                for c in os.listdir(app_config["8A34001_clk_files_path"]):
                    if c.endswith(".tcs"):
                        tcs_files.append(os.path.splitext(c)[0])
                    if c.endswith(".txt"):
                        txt_files.append(os.path.splitext(c)[0])
                    if c.endswith(".bin"):
                        bin_files.append(os.path.splitext(c)[0])
                final_list = list(set(tcs_files+txt_files))
                bin_files = list(set(bin_files))
                upload_tcs_files = []
                upload_txt_files = []
                upload_bin_files = []
                if (os.path.exists(app_config["uploaded_files_path"])):
                    for c in os.listdir(app_config["uploaded_files_path"]):
                        if c.endswith(".tcs"):
                            upload_tcs_files.append(os.path.splitext(c)[0])
                        if c.endswith(".txt"):
                            upload_txt_files.append(os.path.splitext(c)[0])
                        if c.endswith(".bin"):
                            upload_bin_files.append(os.path.splitext(c)[0])
                final_upload_list = list(set(upload_txt_files+upload_tcs_files))
                upload_bin_files = list(set(upload_bin_files))
                resp_json = {
                    "status": "success"
                    , "data": {
                        "default": {
                            "finallist": final_list
                            , "binfiles": bin_files
                        }
                        , "user": {
                            "finaluploadlist": final_upload_list
                            , "binfiles": upload_bin_files
                        }
                    }
                }
                return resp_json, 200
            elif req == "pdi":
                pdifiles = os.listdir(app_config["PDIFilePath"]) if os.path.exists(app_config["PDIFilePath"]) else []
                resp_json = {
                    "status": "success"
                    , "data": {
                        "pdi": {
                            "pdi_files": pdifiles
                        }                        
                    }
                }
                return resp_json,200
            elif req == "rauc":
                raucfiles = os.listdir(app_config["raucFilepath"]) if os.path.exists(app_config["raucFilepath"]) else []
                resp_json = {
                    "status": "success"
                    , "data": {
                        "rauc": {
                            "rauc_files": raucfiles
                        }
                    }
                }
                return resp_json,200
            else:
                resp_json = {
                    "status": "error",
                    "data": {"error": "Invalid function requested"}
                }
                return resp_json, 400
        except Exception as e:
            resp_json = {
                "status":"error"
                ,"data":{"error":"%s"%e}
            }
            print('e',e)
            return resp_json,500

class MultiCmdQuery(Resource):
    def get(self,):
        try:
            if checkJNK() >= 1:
                resp_json = {
                    "status":"error"
                    ,"data": "Notebook kernel is running. Please stop running kernel."
                }
                return resp_json,200

            reqa = request.args.get('sc_cmd')
            ereq = json.loads(reqa)
            tara = request.args.get('target')
            etar = json.loads(tara)
            params_req = request.args.get('params')
            eparams = json.loads(params_req)
            result = {}
            isFail = False
            isSuccess = False
            for i,a in enumerate(ereq):

                req = ereq[i]
                tar = etar[i]
                params = eparams[i].split(",")
                paramStr = eparams[i].replace(","," ")

                try:
                    cmd_gen = sc_app_path+" -c " + req
                    if len(tar):
                        cmd_gen = cmd_gen + " -t '" + tar + "'"
                    if len(params) and len(params[0]):
                        cmd_gen = cmd_gen + " -v '" + paramStr + "'"
                    response = Term.exec_cmd(cmd_gen)
                except Exception as d:
                    print(d)
                if req == "getclock":
                    req_display = "<b>Configured Frequency</b>"
                elif req == "getmeasuredclock":
                    req_display = "<b>Measured Frequency</b>"
                else:
                    req_display = "<b>" + req.capitalize() + "</b>"
                if response.startswith("ERROR:") or "ERROR:" in response:
                    if "error" not in result.keys():
                        result["error"] = ""
                    result["error"] += req_display + " : " + response + "<br>"
                    isFail = True

                else :
                    result1 = parse.parse_cmd_resp(response, req, tar, params)
                    result.update(result1)
                    isSuccess = True
                    if "error" not in result.keys():
                        result["error"] = ""
                    result["error"] += req_display + " : Success \n<br>"
            if isFail and isSuccess:
                resp_json = {
                    "status": "partial_success"
                    , "data": result
                }
                return resp_json,200
            elif isSuccess and not isFail:
                resp_json = {
                    "status": "success"
                    , "data": result
                }
                return resp_json,200
            else:
                resp_json = {
                    "status": "error"
                    , "data": {
                        "message": result["error"]
                    }
                }
                return resp_json, 200
        except Exception as e:
            resp_json = {
                "status":"error"
                , "data":{
                    "message": result["error"]
                }
            }
            return resp_json,500
class CmdQuery(Resource):
    def get(self,):
        try:
            if checkJNK() >= 1: 
                resp_json = { 
                    "status":"error"
                    ,"data": "Notebook kernel is running. Please stop running kernel."
                }                       
                return resp_json,200

            req = request.args.get('sc_cmd')

            tar = request.args.get('target')
            params_req = request.args.get('params')
            params = params_req.split(",")
            paramStr = params_req.replace(","," ")
            
            try:
                cmd_gen = sc_app_path+" -c " + req
                if len(tar):
                    cmd_gen = cmd_gen + " -t '" + tar + "'"
                if len(params) and len(params[0]):
                    cmd_gen = cmd_gen + " -v '" + paramStr + "'"
                response = Term.exec_cmd(cmd_gen)
            except Exception as d:
                print(d)
            if response.startswith("ERROR:") or "ERROR:" in response:
                dresp = {"message":response}
                resp_json = {
                    "status":"error"
                    ,"data":dresp
                }
            else : 
                result = parse.parse_cmd_resp(response, req, tar, params);
                resp_json = {
                    "status":"success"
                    ,"data":result
                }
            if req.startswith("BIT") or "BIT" in req:
                bitLog=SysFactory.exec_cmd("cat "+app_config["bitlogFilePath"],SysFactory.TERMINAL)
                if bitLog.startswith("cat: can't open") or "cat: can't open" in bitLog:
                    resp_json["data"]["bitlogs"] = ""
                else:
                    resp_json["data"]["bitlogs"] = bitLog
            return resp_json,200
        except Exception as e:
            resp_json = {
                "status":"error"
                ,"data":{"error":"%s"%e}
            }
            return resp_json,500
class Notif:
    _notifs = []

    Priority = Enum('Priority',
                    ['PDI'
                     ,'TEMP_RANGE_EXCEED'
                     ])

    MODE_REALTIME = 101            # WHEN NEED TO SHOW/HIDE AT REALTIME. eg: show hide pdi loaded state
    MODE_NETWORK = 102             # WHEN NEED TO SHOW A NETWORK NOTIFICATION. eg: a new board is availabe kind of notification from xilinx web site.
    MODE_INFO = 103                # WHEN NEED TO SHOW FOR A APT UPGRADE IS AVAILABLE. THIS WONT BE REALTIME AND WILL NOT BE DISMISSED ONCE THE CONDITION IS SATISFIED

    TYPE_CMD = 201                  # WHEN THE NOTIFICATION CHECK TYPE IS A LOCAL COMMAND EXECUTION.

    def __init__(self,notif_id="",title="",message="",priority=1000,noti_type=0,type_related_info="",command="",conditionsToCompare=None,req_time="", result = "",prev_req_time="",prev_result = "", mode = 0,show = True):
        self.notif_id = notif_id
        self.title = title
        self.message = message
        self.priority = priority
        self.mode= mode
        self.noti_type= noti_type
        self.type_related_info = type_related_info
        self.command = command
        self.conditionsToCompare = conditionsToCompare
        self.req_time = req_time
        self.result = result
        self.prev_req_time = prev_req_time
        self.prev_result = prev_result
        self.show = show
        Notif._notifs.append(self)
    @staticmethod
    # One time loading list of

    def notification_load():
        Notif(notif_id="PDI_NOT_LOADED"
              , command=sc_app_path + " -c gettemp -t Versal"
              , mode=Notif.MODE_REALTIME
              , noti_type=Notif.TYPE_CMD
              , priority=Notif.Priority.PDI.value
              , conditionsToCompare=lambda x: x.startswith("ERROR: temperature is not available")
              , message="⚠ PDI is not programmed. Ensure to program versal to view temperature value and fan control. Please refer to  <a href='https://xilinx-wiki.atlassian.net/wiki/spaces/A/pages/2273738753/Versal+Evaluation+Board+-+System+Controller#Vivado-Board-Files-%26-PetaLinux-Versal-DUT-BSPs' target='_blank'>wiki</a>"
              )
    @staticmethod
    def jsonObj(noti_ary):
        jsn_ary = []
        for noti in noti_ary:
            jsn_obj = {}
            jsn_obj["notif_id"] = noti.notif_id
            jsn_obj["message"] = noti.message
            jsn_obj["show"] = noti.show
            jsn_ary.append(jsn_obj)
        return jsn_ary

    def notif_check_realTime(self):
        response = Term.exec_cmd(self.command)
        return response

    def validate_notif(self):
        if self.conditionsToCompare(self.result):
            self.show = True
        else:
            self.show = False
        return self.show

    def check_notif(self):
        if self.noti_type == Notif.TYPE_CMD:
            self.prev_result = self.result
            self.result = self.notif_check_realTime()
            return self.validate_notif()

    @staticmethod
    def getNotifs():
        Notif._notifs.sort(key=lambda x: x.priority)
        return Notif._notifs

    @staticmethod
    def identify_highPrior_notif():
        alerts = []
        all_notifs = Notif.getNotifs()
        for notif in all_notifs:
            result = notif.check_notif()
            if result:
                alerts.append(notif)
                # break
        return alerts

    @staticmethod
    def process_notif(open_notifs):
        alert = []
        # check for open_notifs else take the priority one. only one open notification is allowed.
        em_noti = [a_noti for o_noti in open_notifs for a_noti in Notif.getNotifs() if a_noti.notif_id == o_noti["notif_id"]]
        for noti in em_noti:
            if noti.mode == Notif.MODE_REALTIME:
                if noti.check_notif() == False:
                    alert.append(noti)
                break
        else:
            alert = Notif.identify_highPrior_notif()
            for k in em_noti:
                alert.remove(k)
        return alert

class Banner(Resource):
    def get(self, ):
        try:
            result = Notif.process_notif([])
            resp_json = {
                "status": "success"
                , "data": Notif.jsonObj(result)
            }
            return resp_json, 200
        except Exception as e:
            resp_json = {
                "status": "error"
                , "data": {"error": "%s" % e}
            }
            return resp_json, 500
class ScriptRunner(Resource):
    def get(self, ):
        try:
            result = Term.exec_cmd(app_config["scriptfile"])
            resp_jon = {
                "status":"success"
                ,"data": result.strip().split("\n")[-1]
            }
            return resp_jon
        except Exception as e:
            resp_json = {
                "status":"error"
                ,"data":{"error":"%s"%e}
            }
            return resp_json,500
class InstallBoard(Resource):
    def get(self, ):
        try:
            result = Term.exec_cmd(app_config["boardsetupfile"])
            resp_jon = {
                "status":"success"
                ,"data": result.strip().split("\n")[-1]
            }
            return resp_jon
        except Exception as e:
            resp_json = {
                "status":"error"
                ,"data":{"error":"%s"%e}
            }
            return resp_json,500
class exportCSV(Resource):
    def get(self, ):
        try:
            sampling_rate = request.args.get('sampling_rate', 5)
            output_dir = "./static/tmp/"
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            cmd = f"{app_config['csvFIlePath']} -d {sampling_rate}"
            result = Term.exec_cmd(f"python3 {cmd}")
            resp_json = {
                "status":"success"
                ,"data": result.strip().split(" - ")[-1]
            }
            return resp_json
        except Exception as e:
            resp_json = {
                "status":"error"
                ,"data":{"error":"%s"%e}
            }
            return resp_json,500
class RaucUpdate(Resource):
    def get(self, ):
        try:
            req = request.args.get('func')
            tar = request.args.get('target')
            if req == "reboot":
                cmd_gen = req
            else:
                cmd_gen = "rauc " + req
            if len(tar):
                cmd_gen = cmd_gen + " -t '" + tar + "'"
            result = Term.exec_cmd(cmd_gen)
            if "ERROR:" in result or "failed" in result or "No such file" in result:
                resp_json = {
                    "status": "error"
                    , "data": result
                }
                return resp_json
            else:
                resp_json = {
                    "status": "success"
                    , "data": result
                }
                return resp_json
        except Exception as e:
            resp_json = {
                "status": "error"
                , "data": {"error": "%s" % e}
            }
            return resp_json, 500
